/*
 * ${license.title}
 * Copyright (C) 2022 ${license.owner}
 * ${license.mailto}
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

//
// Created by webpigeon on 23/04/22.
//

#ifndef DEMO_ROLLBALL_HPP
#define DEMO_ROLLBALL_HPP

#include "fggl/scenes/game.hpp"
#include "fggl/phys/service.hpp"
#include "fggl/script/engine.hpp"

namespace demo {

	enum class DebugMode {
			NORMAL = 0,
			DISTANCE = 1,
			VISION = 2
	};

	struct RollState {
		fggl::entity::EntityID player = fggl::entity::INVALID;
		fggl::entity::EntityID closestPickup;
		DebugMode mode = DebugMode::NORMAL;

		std::array<fggl::entity::EntityID, 3> collectables {
			fggl::entity::INVALID,
			fggl::entity::INVALID,
			fggl::entity::INVALID };
		float time = 0.0f;
	};

	class RollBall : public fggl::scenes::Game {

		public:

			explicit RollBall(fggl::App& app);

			void activate() override;
			void deactivate() override;

			void update(float dt) override;
			void render(fggl::gfx::Graphics& gfx) override;

		private:
			constexpr static fggl::math::vec3 HINT_COLOUR{0.5f, 0.0f, 0.0f};
			RollState state;
			fggl::phys::PhysicsEngine* m_phys;
			fggl::script::ScriptEngine* m_scripts;
			fggl::math::vec3 cameraOffset = {-15.0F, 15.0F, 0.0F};

			void closestPickup(fggl::entity::EntityManager& world);
			void spinCubes(fggl::entity::EntityManager& world, float dt);
	};

}

#endif //DEMO_ROLLBALL_HPP
